import os
import sys
import importlib
import inspect
import falcon
from netq_backend.modules import node

from netq_backend.cmd.urls import get_urls
from netq_lib.common import configuration
from netq_lib.common.enums import ConfKw
from netq_lib.orm.redisdb import set_connection_settings, BackendExceptions, Connection
from netq_lib.common import utils
from netq_apps.lib.netq import NetQ

_PROC_NAME = "netq-backend"

conf_dict = configuration.get_config(_PROC_NAME, True)
logger = utils.get_logger(_PROC_NAME, conf_dict)
lib_netq = NetQ(None, None, None, logger)
netq = None

def add_route(app):
    urls = get_urls()
    for url, cls in urls.iteritems():
        app.add_route(url, cls)


#
# Create a Falcon API end
#
app = application = falcon.API()

app.add_route('/webapi/v1/nodes', node.WebApiNodeAll(lib_netq, logger))
app.add_route('/webapi/v1/ping', node.WebApiPing(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}', node.WebApiNode(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/inventory', node.WebApiNodeInventory(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/sensor', node.WebApiNodeSensors(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/port', node.WebApiNodePorts(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/ntp', node.WebApiNodeNtp(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/service', node.WebApiNodeServices(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/bgp', node.WebApiNodeBgp(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/interface', node.WebApiNodeInterfaces(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/lldp', node.WebApiNodeLldp(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/vlan', node.WebApiNodeVlans(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/evpn', node.WebApiNodeEvpn(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/mac', node.WebApiNodeMacs(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/agent', node.WebApiNodeAgents(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/clag', node.WebApiNodeClag(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/ospf', node.WebApiNodeOspf(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/vxlan', node.WebApiNodeVxlans(lib_netq, logger))
app.add_route('/webapi/v1/nodes/{hostname}/address', node.WebApiNodeAddresses(lib_netq, logger))

#
# Add URL paths 
#
add_route(app)
