import falcon
import json
from netq_apps.modules.core.platform import inventory
from netq_apps.modules.core.netq import agents
from netq_apps.modules.core.platform import sensors
from netq_apps.modules.core.platform import cables
from netq_apps.modules.core.os import ntp
from netq_apps.modules.core.os import services
from netq_apps.modules.core.l3 import bgp
from netq_apps.modules.core.l3 import ospf
from netq_apps.modules.core.ip import link
from netq_apps.modules.core.l2 import vlan
from netq_apps.modules.core.l2 import mac
from netq_apps.modules.core.l2 import lldp
from netq_apps.modules.core.l2 import clag
from netq_apps.modules.core.nvo import evpn
from netq_apps.modules.core.nvo import vxlan
from netq_apps.modules.core.ip import address



class WebApiBase(object):

    def __init__(self, lib_netq, logger):
        self._lib_netq = lib_netq
        self._logger = logger

    def get_host_sensors(self, hostname):
        sensor_dict = {}
        for ele in sensors.get_sensor_status(self._lib_netq, hostname, '*', False, 'Temp'):
            if not sensor_dict.get('Temp'):
                sensor_dict['Temp'] = []
            sensor_dict['Temp'].append(ele)
        for ele in sensors.get_sensor_status(self._lib_netq, hostname, '*', False, 'Fan'):
            if not sensor_dict.get('Fan'):
                sensor_dict['Fan'] = []
            sensor_dict['Fan'].append(ele)
        for ele in sensors.get_sensor_status(self._lib_netq, hostname, '*', False, 'PSU'):
            if not sensor_dict.get('PSU'):
                sensor_dict['PSU'] = []
            sensor_dict['PSU'].append(ele)
        return sensor_dict

    def get_host_inventory(self, hostname):
        inventory_map = {
            "ASIC": inventory.get_asic_status(self._lib_netq, hostname, None,
                                              None, None),
            "Board": inventory.get_board_status(self._lib_netq, hostname, None,
                                                None),
            "CPU": inventory.get_cpu_status(self._lib_netq, hostname, None),
            "Disk": inventory.get_disk_status(self._lib_netq, hostname, None,
                                              None, None),
            "Memory": inventory.get_memory_status(self._lib_netq, hostname,
                                                  None, None),
            "License": inventory.get_license_status(self._lib_netq, hostname,
                                                    '*', None, False),
            "OS": inventory.get_os_status(self._lib_netq, hostname, None, None,
                                          False)}

        inventory_dict = {}
        for inv_type, inv_gen in inventory_map.items():
            for ele in inv_gen:
                inventory_dict[inv_type] = ele
        return inventory_dict

    def get_host_ports(self, hostname):
        ports_list = []
        for ele in cables.get_physical_ports(
                self._lib_netq, hostname=hostname, ifname="*", empty=False,
                plugged=False, vendor=None, model=None, module=None,
                changes=False):
            ports_list.append(ele)
        return ports_list

    def get_host_ntp(self, hostname):
        ntp_dict = {}
        for ele in ntp.get_ntp_status(self._lib_netq, jsonify=False, changes=False,
                                      hostname=hostname, sync_state='*'):
            ntp_dict = ele
        return ntp_dict

    def get_host_services(self, hostname):
        services_list = []
        for ele in services.get_services_status(self._lib_netq, hostname=hostname):
            services_list.append(ele)
        return services_list

    def get_host_bgp(self, hostname):
        bgp_list = []
        for ele in bgp.get_bgp_status_details(self._lib_netq, hostname, '*', '*', '*', False):
            bgp_list.append(ele)
        return bgp_list

    def get_host_interfaces(self, hostname):
        interfaces_list = []
        for ele in link.get_links(self._lib_netq, hostname, '*', '*', None, False):
            interfaces_list.append(ele)
        return interfaces_list

    def get_host_vlans(self, hostname):
        vlans_list = []
        for ele in vlan.get_vlan_details(self._lib_netq, hostname=hostname):
            vlans_list.append(ele)
        return vlans_list

    def get_host_evpn(self, hostname):
        evpn_list = []
        for ele in evpn.get_evpn_details(self._lib_netq, hostname=hostname):
            evpn_list.append(ele)
        return evpn_list

    def get_host_macs(self, hostname):
        macs_list = []
        for ele in mac.get_macs(self._lib_netq, hostname=hostname):
            macs_list.append(ele)
        return macs_list

    def get_host_lldp(self, hostname):
        lldp_list = []
        for ele in lldp.get_lldp_status(self._lib_netq, hostname, '*', False):
            lldp_list.append(ele)
        return lldp_list

    def get_host_agent(self, hostname):
        for ele in agents.show_status(self._lib_netq, hostname, False):
            return ele
        return dict()

    def get_host_clag(self, hostname):
        clag_list = []
        for ele in clag.get_clag_status_details(self._lib_netq, hostname=hostname, changes=False):
            clag_list.append(ele)
        return clag_list

    def get_host_ospf(self, hostname):
        # get_ospf_status(netq, hostname, ifname, area, changes)
        ospf_list = []
        for ele in ospf.get_ospf_status(self._lib_netq, hostname, '*', '*', False):
            ospf_list.append(ele)
        return ospf_list

    def get_host_vxlans(self, hostname):
        vxlan_list = []
        for ele in vxlan.get_vxlan_details(self._lib_netq, hostname=hostname, vni='*', changes=False):
            vxlan_list.append(ele)
        return vxlan_list

    def get_host_addresses(self, hostname):
        address_list = []
        for ele in address.get_addresses(self._lib_netq, hostname=hostname):
            address_list.append(ele)
        return address_list

    def get_host_data(self, hostname):
        host_dict = {
                'Inventory': self.get_host_inventory(hostname),
                'Sensor': self.get_host_sensors(hostname),
                'Port': self.get_host_ports(hostname),
                'NTP': self.get_host_ntp(hostname),
                'Service': self.get_host_services(hostname),
                'BGP': self.get_host_bgp(hostname),
                'Interface': self.get_host_interfaces(hostname),
                'VLAN': self.get_host_vlans(hostname),
                'VXLAN': self.get_host_vxlans(hostname),
                'EVPN': self.get_host_evpn(hostname),
                'MAC': self.get_host_macs(hostname),
                'LLDP': self.get_host_lldp(hostname),
                'Agent': self.get_host_agent(hostname),
                'CLAG': self.get_host_clag(hostname),
                'OSPF': self.get_host_ospf(hostname),
                'Address': self.get_host_addresses(hostname),
                }
        return host_dict

    def get_hosts_data(self, hostname='*'):
        nodes_dict_list = list()
        for ele in agents.show_status(self._lib_netq, '*', False):
            nodes_dict_list.append(self.get_host_data(ele["Hostname"]))
        return nodes_dict_list

    def make_resp(self, req, resp, hostname, func):
        try:
            data = func(hostname)
            resp.body = json.dumps({'data': data},
                                   ensure_ascii=False)
        except BaseException as ex:
            raise falcon.HTTPInternalServerError(
                title='Internal Server Error', description=ex)


class WebApiNode(WebApiBase):

    def on_get(self, req, resp, hostname='*'):
        self.make_resp(req, resp, hostname, self.get_host_data)


class WebApiNodeAll(WebApiBase):

    def on_get(self, req, resp, hostname='*'):
        self.make_resp(req, resp, hostname, self.get_hosts_data)


class WebApiNodeInventory(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_inventory)


class WebApiNodeSensors(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_sensors)


class WebApiNodePorts(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_ports)


class WebApiNodeNtp(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_ntp)


class WebApiNodeServices(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_services)


class WebApiNodeBgp(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_bgp)


class WebApiNodeInterfaces(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_interfaces)


class WebApiNodeLldp(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_lldp)


class WebApiNodeVlans(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_vlans)


class WebApiNodeEvpn(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_evpn)


class WebApiNodeMacs(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_macs)


class WebApiNodeAgents(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_agent)


class WebApiNodeClag(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_clag)


class WebApiNodeOspf(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_ospf)


class WebApiNodeVxlans(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_vxlans)


class WebApiNodeAddresses(WebApiBase):

    def on_get(self, req, resp, hostname):
        self.make_resp(req, resp, hostname, self.get_host_addresses)


class WebApiPing(WebApiBase):

    def on_get(self, req, resp):
        resp.body = json.dumps({'data': 'pong'}, ensure_ascii=False)
