from __future__ import absolute_import
import falcon
import argparse
import time
import sys
import json

from subprocess import CalledProcessError, check_output, STDOUT
from netq_backend.lib.parser import (
    get_request_data,
)


class NetqQuery(object):

    def parse_request(self, req_data):
        self.query_string = req_data.get('query', None)
        if self.query_string:
            self.query_string = str(self.query_string)
        return

    def on_post(self, req, resp):
        '''
        Expect request format:
            {
               'query': <query string>
            }
        '''
        resp_data = {
            'request': None,
            'error': None,
            'netq': None,
            'retcode': 0,
        }
        resp.status = falcon.HTTP_200
        resp.content_type = 'application/json'

        try:
            req_data = get_request_data(req)
        except Exception as e:
            resp.status = falcon.HTTP_400
            resp_data['error'] = 'Invalid POST data'
            resp_data['retcode'] = 400
            resp.data = json.dumps(resp_data)
            return

        self.parse_request(req_data)
        resp_data['request'] = req_data

        if not self.query_string:
            resp.status = falcon.HTTP_400
            resp.content_type = 'application/json'
            resp_data['error'] = 'Invalid POST data'
            resp_data['retcode'] = 400
            resp.data = json.dumps(resp_data)
            return

        if self.query_string.startswith('netq'):
            output = None
            try:
                output = check_output(('{0}'.format(self.query_string)).split(), stderr=STDOUT)
            except CalledProcessError as e:
                resp_data['error'] = 'Command failed with error:  {}'.format(e)
                output = e.output
                resp_data['retcode'] = e.returncode
            except Exception as e:
                resp_data['error'] = 'Request failed due to {}'.format(e)
                resp_data['retcode'] = 400
            if output:
                if "json" in self.query_string:
                    resp_data['netq'] = json.loads(output)
                else:
                    resp_data['netq'] = output
        else:
            resp.status = falcon.HTTP_400
            resp_data['error'] = "Invalid Query. Only 'netq ...' commands are supported"
            resp_data['retcode'] = 400
            resp.data = json.dumps(resp_data)
            return

        resp.data = json.dumps(resp_data)
        return
